@@ -28,9 +28,7 @@
  *	    Chris Wilson <chris@chris-wilson.co.uk>
  */
 
-#define _GNU_SOURCE 1	/* for sched_getaffinity() and getline() */
-
-#include "../cairo-version.h" /* for the real version */
+#include "config.h"
 
 #include "cairo-missing.h"
 #include "cairo-perf.h"
@@ -199,7 +197,6 @@ scache_equal (const void *A,
     return a->entry.hash == b->entry.hash;
 }
 
-#define ARRAY_SIZE(A) (sizeof(A)/sizeof(A[0]))
 static void
 scache_mark_active (cairo_surface_t *surface)
 {
@@ -210,7 +207,7 @@ scache_mark_active (cairo_surface_t *surface)
 	return;
 
     t0 = cairo_surface_reference (surface);
-    for (n = 0; n < ARRAY_SIZE (surface_holdovers); n++) {
+    for (n = 0; n < ARRAY_LENGTH (surface_holdovers); n++) {
 	if (surface_holdovers[n] == surface) {
 	    surface_holdovers[n] = t0;
 	    t0 = surface;
@@ -232,7 +229,7 @@ scache_clear (void)
     if (surface_cache == NULL)
 	return;
 
-    for (n = 0; n < ARRAY_SIZE (surface_holdovers); n++) {
+    for (n = 0; n < ARRAY_LENGTH (surface_holdovers); n++) {
 	cairo_surface_destroy (surface_holdovers[n]);
 	surface_holdovers[n] = NULL;
     }
@@ -378,7 +375,7 @@ usage (const char *argv0)
 "  -i	iterations; specify the number of iterations per test case\n"
 "  -l	list only; just list selected test case names without executing\n"
 "  -r	raw; display each time measurement instead of summary statistics\n"
-"  -s	sync; only sum the elapsed time of the indiviual operations\n"
+"  -s	sync; only sum the elapsed time of the individual operations\n"
 "  -t	tile size; draw to tiled surfaces\n"
 "  -v	verbose; in raw mode also show the summaries\n"
 "  -x	exclude; specify a file to read a list of traces to exclude\n"
@@ -410,11 +407,11 @@ read_excludes (cairo_perf_t *perf,
 
 	/* whitespace delimits */
 	s = line;
-	while (*s != '\0' && isspace (*s))
+	while (*s != '\0' && isspace ((unsigned char)*s))
 	    s++;
 
 	t = s;
-	while (*t != '\0' && ! isspace (*t))
+	while (*t != '\0' && ! isspace ((unsigned char)*t))
 	    t++;
 
 	if (s != t) {
