@@ -45,14 +45,94 @@
 
 #include <assert.h>
 
-/* The autoconf on OpenBSD 4.5 produces the malformed constant name
- * SIZEOF_VOID__ rather than SIZEOF_VOID_P.  Work around that here. */
-#if !defined(SIZEOF_VOID_P) && defined(SIZEOF_VOID__)
-# define SIZEOF_VOID_P SIZEOF_VOID__
-#endif
-
 CAIRO_BEGIN_DECLS
 
+#if HAVE_C11_ATOMIC_PRIMITIVES
+
+#include <stdatomic.h>
+
+#define HAS_ATOMIC_OPS 1
+
+typedef atomic_int cairo_atomic_int_t;
+typedef _Atomic(void *) cairo_atomic_intptr_t;
+
+static cairo_always_inline int
+_cairo_atomic_int_get (cairo_atomic_int_t *x)
+{
+    return atomic_load_explicit (x, memory_order_seq_cst);
+}
+
+static cairo_always_inline int
+_cairo_atomic_int_get_relaxed (cairo_atomic_int_t *x)
+{
+    return atomic_load_explicit (x, memory_order_relaxed);
+}
+
+static cairo_always_inline void
+_cairo_atomic_int_set_relaxed (cairo_atomic_int_t *x, int val)
+{
+    atomic_store_explicit (x, val, memory_order_relaxed);
+}
+
+static cairo_always_inline void *
+_cairo_atomic_ptr_get (cairo_atomic_intptr_t *x)
+{
+    return atomic_load_explicit (x, memory_order_seq_cst);
+}
+
+# define _cairo_atomic_int_inc(x) ((void) atomic_fetch_add_explicit(x, 1, memory_order_seq_cst))
+# define _cairo_atomic_int_dec(x) ((void) atomic_fetch_sub_explicit(x, 1, memory_order_seq_cst))
+# define _cairo_atomic_int_dec_and_test(x) (atomic_fetch_sub_explicit(x, 1, memory_order_seq_cst) == 1)
+
+
+static cairo_always_inline cairo_bool_t
+_cairo_atomic_int_cmpxchg_impl(cairo_atomic_int_t *x,
+			       int                oldv,
+			       int                newv)
+{
+    int expected = oldv;
+    return atomic_compare_exchange_strong_explicit (x, &expected, newv, memory_order_seq_cst, memory_order_seq_cst);
+}
+
+#define _cairo_atomic_int_cmpxchg(x, oldv, newv) \
+  _cairo_atomic_int_cmpxchg_impl(x, oldv, newv)
+
+static cairo_always_inline int
+_cairo_atomic_int_cmpxchg_return_old_impl(cairo_atomic_int_t *x,
+					  int                 oldv,
+					  int                 newv)
+{
+    int expected = oldv;
+    (void) atomic_compare_exchange_strong_explicit (x, &expected, newv, memory_order_seq_cst, memory_order_seq_cst);
+    return expected;
+}
+
+#define _cairo_atomic_int_cmpxchg_return_old(x, oldv, newv) \
+  _cairo_atomic_int_cmpxchg_return_old_impl(x, oldv, newv)
+
+static cairo_always_inline cairo_bool_t
+_cairo_atomic_ptr_cmpxchg_impl(cairo_atomic_intptr_t *x, void *oldv, void *newv)
+{
+    void *expected = oldv;
+    return atomic_compare_exchange_strong_explicit (x, &expected, newv, memory_order_seq_cst, memory_order_seq_cst);
+}
+
+#define _cairo_atomic_ptr_cmpxchg(x, oldv, newv) \
+  _cairo_atomic_ptr_cmpxchg_impl(x, oldv, newv)
+
+static cairo_always_inline void *
+_cairo_atomic_ptr_cmpxchg_return_old_impl(cairo_atomic_intptr_t *x, void *oldv, void *newv)
+{
+    void *expected = oldv;
+    (void) atomic_compare_exchange_strong_explicit (x, &expected, newv, memory_order_seq_cst, memory_order_seq_cst);
+    return expected;
+}
+
+#define _cairo_atomic_ptr_cmpxchg_return_old(x, oldv, newv) \
+  _cairo_atomic_ptr_cmpxchg_return_old_impl(x, oldv, newv)
+
+#endif /* HAVE_C11_ATOMIC_PRIMITIVES */
+
 /* C++11 atomic primitives were designed to be more flexible than the
  * __sync_* family of primitives.  Despite the name, they are available
  * in C as well as C++.  The motivating reason for using them is that
@@ -68,63 +148,54 @@ CAIRO_BEGIN_DECLS
 #define HAS_ATOMIC_OPS 1
 
 typedef int cairo_atomic_int_t;
+typedef intptr_t cairo_atomic_intptr_t;
 
-static cairo_always_inline cairo_atomic_int_t
+static cairo_always_inline int
 _cairo_atomic_int_get (cairo_atomic_int_t *x)
 {
     return __atomic_load_n(x, __ATOMIC_SEQ_CST);
 }
 
-static cairo_always_inline cairo_atomic_int_t
+static cairo_always_inline int
 _cairo_atomic_int_get_relaxed (cairo_atomic_int_t *x)
 {
     return __atomic_load_n(x, __ATOMIC_RELAXED);
 }
 
 static cairo_always_inline void
-_cairo_atomic_int_set_relaxed (cairo_atomic_int_t *x, cairo_atomic_int_t val)
+_cairo_atomic_int_set_relaxed (cairo_atomic_int_t *x, int val)
 {
     __atomic_store_n(x, val, __ATOMIC_RELAXED);
 }
 
 static cairo_always_inline void *
-_cairo_atomic_ptr_get (void **x)
+_cairo_atomic_ptr_get (cairo_atomic_intptr_t *x)
 {
-    return __atomic_load_n(x, __ATOMIC_SEQ_CST);
+    return (void*)__atomic_load_n(x, __ATOMIC_SEQ_CST);
 }
 
 # define _cairo_atomic_int_inc(x) ((void) __atomic_fetch_add(x, 1, __ATOMIC_SEQ_CST))
 # define _cairo_atomic_int_dec(x) ((void) __atomic_fetch_sub(x, 1, __ATOMIC_SEQ_CST))
 # define _cairo_atomic_int_dec_and_test(x) (__atomic_fetch_sub(x, 1, __ATOMIC_SEQ_CST) == 1)
 
-#if SIZEOF_VOID_P==SIZEOF_INT
-typedef int cairo_atomic_intptr_t;
-#elif SIZEOF_VOID_P==SIZEOF_LONG
-typedef long cairo_atomic_intptr_t;
-#elif SIZEOF_VOID_P==SIZEOF_LONG_LONG
-typedef long long cairo_atomic_intptr_t;
-#else
-#error No matching integer pointer type
-#endif
-
 static cairo_always_inline cairo_bool_t
 _cairo_atomic_int_cmpxchg_impl(cairo_atomic_int_t *x,
-			       cairo_atomic_int_t oldv,
-			       cairo_atomic_int_t newv)
+			       int                 oldv,
+			       int                 newv)
 {
-    cairo_atomic_int_t expected = oldv;
+    int expected = oldv;
     return __atomic_compare_exchange_n(x, &expected, newv, 0, __ATOMIC_SEQ_CST, __ATOMIC_SEQ_CST);
 }
 
 #define _cairo_atomic_int_cmpxchg(x, oldv, newv) \
   _cairo_atomic_int_cmpxchg_impl(x, oldv, newv)
 
-static cairo_always_inline cairo_atomic_int_t
+static cairo_always_inline int
 _cairo_atomic_int_cmpxchg_return_old_impl(cairo_atomic_int_t *x,
-					  cairo_atomic_int_t oldv,
-					  cairo_atomic_int_t newv)
+					  int                 oldv,
+					  int                 newv)
 {
-    cairo_atomic_int_t expected = oldv;
+    int expected = oldv;
     (void) __atomic_compare_exchange_n(x, &expected, newv, 0, __ATOMIC_SEQ_CST, __ATOMIC_SEQ_CST);
     return expected;
 }
@@ -133,58 +204,59 @@ _cairo_atomic_int_cmpxchg_return_old_impl(cairo_atomic_int_t *x,
   _cairo_atomic_int_cmpxchg_return_old_impl(x, oldv, newv)
 
 static cairo_always_inline cairo_bool_t
-_cairo_atomic_ptr_cmpxchg_impl(void **x, void *oldv, void *newv)
+_cairo_atomic_ptr_cmpxchg_impl(cairo_atomic_intptr_t *x, void *oldv, void *newv)
 {
-    void *expected = oldv;
-    return __atomic_compare_exchange_n(x, &expected, newv, 0, __ATOMIC_SEQ_CST, __ATOMIC_SEQ_CST);
+    intptr_t expected = (intptr_t)oldv;
+    return __atomic_compare_exchange_n(x, &expected, (intptr_t)newv, 0, __ATOMIC_SEQ_CST, __ATOMIC_SEQ_CST);
 }
 
 #define _cairo_atomic_ptr_cmpxchg(x, oldv, newv) \
   _cairo_atomic_ptr_cmpxchg_impl(x, oldv, newv)
 
 static cairo_always_inline void *
-_cairo_atomic_ptr_cmpxchg_return_old_impl(void **x, void *oldv, void *newv)
+_cairo_atomic_ptr_cmpxchg_return_old_impl(cairo_atomic_intptr_t *x, void *oldv, void *newv)
 {
-    void *expected = oldv;
-    (void) __atomic_compare_exchange_n(x, &expected, newv, 0, __ATOMIC_SEQ_CST, __ATOMIC_SEQ_CST);
-    return expected;
+    intptr_t expected = (intptr_t)oldv;
+    (void) __atomic_compare_exchange_n(x, &expected, (intptr_t)newv, 0, __ATOMIC_SEQ_CST, __ATOMIC_SEQ_CST);
+    return (void*)expected;
 }
 
 #define _cairo_atomic_ptr_cmpxchg_return_old(x, oldv, newv) \
   _cairo_atomic_ptr_cmpxchg_return_old_impl(x, oldv, newv)
 
-#endif
+#endif /* HAVE_CXX11_ATOMIC_PRIMITIVES */
 
 #if HAVE_GCC_LEGACY_ATOMICS
 
 #define HAS_ATOMIC_OPS 1
 
 typedef int cairo_atomic_int_t;
+typedef intptr_t cairo_atomic_intptr_t;
 
-static cairo_always_inline cairo_atomic_int_t
+static cairo_always_inline int
 _cairo_atomic_int_get (cairo_atomic_int_t *x)
 {
     __sync_synchronize ();
     return *x;
 }
 
-static cairo_always_inline cairo_atomic_int_t
+static cairo_always_inline int
 _cairo_atomic_int_get_relaxed (cairo_atomic_int_t *x)
 {
     return *x;
 }
 
 static cairo_always_inline void
-_cairo_atomic_int_set_relaxed (cairo_atomic_int_t *x, cairo_atomic_int_t val)
+_cairo_atomic_int_set_relaxed (cairo_atomic_int_t *x, int val)
 {
     *x = val;
 }
 
 static cairo_always_inline void *
-_cairo_atomic_ptr_get (void **x)
+_cairo_atomic_ptr_get (cairo_atomic_intptr_t *x)
 {
     __sync_synchronize ();
-    return *x;
+    return (void*)*x;
 }
 
 # define _cairo_atomic_int_inc(x) ((void) __sync_fetch_and_add(x, 1))
@@ -193,23 +265,13 @@ _cairo_atomic_ptr_get (void **x)
 # define _cairo_atomic_int_cmpxchg(x, oldv, newv) __sync_bool_compare_and_swap (x, oldv, newv)
 # define _cairo_atomic_int_cmpxchg_return_old(x, oldv, newv) __sync_val_compare_and_swap (x, oldv, newv)
 
-#if SIZEOF_VOID_P==SIZEOF_INT
-typedef int cairo_atomic_intptr_t;
-#elif SIZEOF_VOID_P==SIZEOF_LONG
-typedef long cairo_atomic_intptr_t;
-#elif SIZEOF_VOID_P==SIZEOF_LONG_LONG
-typedef long long cairo_atomic_intptr_t;
-#else
-#error No matching integer pointer type
-#endif
-
 # define _cairo_atomic_ptr_cmpxchg(x, oldv, newv) \
     __sync_bool_compare_and_swap ((cairo_atomic_intptr_t*)x, (cairo_atomic_intptr_t)oldv, (cairo_atomic_intptr_t)newv)
 
 # define _cairo_atomic_ptr_cmpxchg_return_old(x, oldv, newv) \
     _cairo_atomic_intptr_to_voidptr (__sync_val_compare_and_swap ((cairo_atomic_intptr_t*)x, (cairo_atomic_intptr_t)oldv, (cairo_atomic_intptr_t)newv))
 
-#endif
+#endif /* HAVE_GCC_LEGACY_ATOMICS */
 
 #if HAVE_LIB_ATOMIC_OPS
 #include <atomic_ops.h>
@@ -227,15 +289,7 @@ typedef  AO_t cairo_atomic_int_t;
 # define _cairo_atomic_int_dec_and_test(x) (AO_fetch_and_sub1_full(x) == 1)
 # define _cairo_atomic_int_cmpxchg(x, oldv, newv) AO_compare_and_swap_full(x, oldv, newv)
 
-#if SIZEOF_VOID_P==SIZEOF_INT
-typedef unsigned int cairo_atomic_intptr_t;
-#elif SIZEOF_VOID_P==SIZEOF_LONG
-typedef unsigned long cairo_atomic_intptr_t;
-#elif SIZEOF_VOID_P==SIZEOF_LONG_LONG
-typedef unsigned long long cairo_atomic_intptr_t;
-#else
-#error No matching integer pointer type
-#endif
+typedef intptr_t cairo_atomic_intptr_t;
 
 # define _cairo_atomic_ptr_get(x) _cairo_atomic_intptr_to_voidptr (AO_load_full (x))
 # define _cairo_atomic_ptr_cmpxchg(x, oldv, newv) \
@@ -249,6 +303,7 @@ typedef unsigned long long cairo_atomic_intptr_t;
 #define HAS_ATOMIC_OPS 1
 
 typedef int32_t cairo_atomic_int_t;
+typedef intptr_t cairo_atomic_intptr_t;
 
 # define _cairo_atomic_int_get(x) (OSMemoryBarrier(), *(x))
 # define _cairo_atomic_int_get_relaxed(x) *(x)
@@ -260,12 +315,10 @@ typedef int32_t cairo_atomic_int_t;
 # define _cairo_atomic_int_cmpxchg(x, oldv, newv) OSAtomicCompareAndSwap32Barrier(oldv, newv, x)
 
 #if SIZEOF_VOID_P==4
-typedef int32_t cairo_atomic_intptr_t;
 # define _cairo_atomic_ptr_cmpxchg(x, oldv, newv) \
     OSAtomicCompareAndSwap32Barrier((cairo_atomic_intptr_t)oldv, (cairo_atomic_intptr_t)newv, (cairo_atomic_intptr_t *)x)
 
 #elif SIZEOF_VOID_P==8
-typedef int64_t cairo_atomic_intptr_t;
 # define _cairo_atomic_ptr_cmpxchg(x, oldv, newv) \
     OSAtomicCompareAndSwap64Barrier((cairo_atomic_intptr_t)oldv, (cairo_atomic_intptr_t)newv, (cairo_atomic_intptr_t *)x)
 
@@ -275,26 +328,17 @@ typedef int64_t cairo_atomic_intptr_t;
 
 # define _cairo_atomic_ptr_get(x) (OSMemoryBarrier(), *(x))
 
-#endif
+#endif /* HAVE_OS_ATOMIC_OPS */
 
 #if !defined(HAS_ATOMIC_OPS) && defined(_WIN32)
 #include <windows.h>
 
 #define HAS_ATOMIC_OPS 1
 
-typedef int32_t cairo_atomic_int_t;
+typedef LONG cairo_atomic_int_t;
+typedef PVOID cairo_atomic_intptr_t;
 
-#if SIZEOF_VOID_P==SIZEOF_INT
-typedef unsigned int cairo_atomic_intptr_t;
-#elif SIZEOF_VOID_P==SIZEOF_LONG
-typedef unsigned long cairo_atomic_intptr_t;
-#elif SIZEOF_VOID_P==SIZEOF_LONG_LONG
-typedef unsigned long long cairo_atomic_intptr_t;
-#else
-#error No matching integer pointer type
-#endif
-
-static cairo_always_inline cairo_atomic_int_t
+static cairo_always_inline int
 _cairo_atomic_int_get (cairo_atomic_int_t *x)
 {
     MemoryBarrier ();
@@ -310,27 +354,27 @@ _cairo_atomic_int_get (cairo_atomic_int_t *x)
 
 static cairo_always_inline cairo_bool_t
 _cairo_atomic_int_cmpxchg (cairo_atomic_int_t *x,
-                           cairo_atomic_int_t oldv,
-                           cairo_atomic_int_t newv)
+                           int                 oldv,
+                           int                 newv)
 {
-    return InterlockedCompareExchange ((unsigned int*)x, (unsigned int)newv, (unsigned int)oldv) == oldv;
+    return InterlockedCompareExchange (x, (LONG)newv, (LONG)oldv) == oldv;
 }
 
 static cairo_always_inline void *
-_cairo_atomic_ptr_get (void **x)
+_cairo_atomic_ptr_get (cairo_atomic_intptr_t *x)
 {
     MemoryBarrier ();
-    return *x;
+    return (void *) *x;
 }
 
 static cairo_always_inline cairo_bool_t
-_cairo_atomic_ptr_cmpxchg (void **x, void *oldv, void *newv)
+_cairo_atomic_ptr_cmpxchg (cairo_atomic_intptr_t *x, void *oldv, void *newv)
 {
     return InterlockedCompareExchangePointer (x, newv, oldv) == oldv;
 }
 
 static cairo_always_inline void *
-_cairo_atomic_ptr_cmpxchg_return_old (void **x, void *oldv, void *newv)
+_cairo_atomic_ptr_cmpxchg_return_old (cairo_atomic_intptr_t *x, void *oldv, void *newv)
 {
     return InterlockedCompareExchangePointer (x, newv, oldv);
 }
@@ -340,17 +384,8 @@ _cairo_atomic_ptr_cmpxchg_return_old (void **x, void *oldv, void *newv)
 
 #ifndef HAS_ATOMIC_OPS
 
-#if SIZEOF_VOID_P==SIZEOF_INT
-typedef unsigned int cairo_atomic_intptr_t;
-#elif SIZEOF_VOID_P==SIZEOF_LONG
-typedef unsigned long cairo_atomic_intptr_t;
-#elif SIZEOF_VOID_P==SIZEOF_LONG_LONG
-typedef unsigned long long cairo_atomic_intptr_t;
-#else
-#error No matching integer pointer type
-#endif
-
-typedef cairo_atomic_intptr_t cairo_atomic_int_t;
+typedef int cairo_atomic_int_t;
+typedef intptr_t cairo_atomic_intptr_t;
 
 cairo_private void
 _cairo_atomic_int_inc (cairo_atomic_int_t *x);
@@ -360,23 +395,24 @@ _cairo_atomic_int_inc (cairo_atomic_int_t *x);
 cairo_private cairo_bool_t
 _cairo_atomic_int_dec_and_test (cairo_atomic_int_t *x);
 
-cairo_private cairo_atomic_int_t
-_cairo_atomic_int_cmpxchg_return_old_impl (cairo_atomic_int_t *x, cairo_atomic_int_t oldv, cairo_atomic_int_t newv);
+cairo_private int
+_cairo_atomic_int_cmpxchg_return_old_impl (cairo_atomic_int_t *x, int oldv, int newv);
 
 cairo_private void *
-_cairo_atomic_ptr_cmpxchg_return_old_impl (void **x, void *oldv, void *newv);
+_cairo_atomic_ptr_cmpxchg_return_old_impl (cairo_atomic_intptr_t *x, void *oldv, void *newv);
 
 #define _cairo_atomic_int_cmpxchg_return_old(x, oldv, newv) _cairo_atomic_int_cmpxchg_return_old_impl (x, oldv, newv)
 #define _cairo_atomic_ptr_cmpxchg_return_old(x, oldv, newv) _cairo_atomic_ptr_cmpxchg_return_old_impl (x, oldv, newv)
 
 #ifdef ATOMIC_OP_NEEDS_MEMORY_BARRIER
-cairo_private cairo_atomic_int_t
+cairo_private int
 _cairo_atomic_int_get (cairo_atomic_int_t *x);
-cairo_private cairo_atomic_int_t
+cairo_private int
 _cairo_atomic_int_get_relaxed (cairo_atomic_int_t *x);
 void
-_cairo_atomic_int_set_relaxed (cairo_atomic_int_t *x, cairo_atomic_int_t val);
-# define _cairo_atomic_ptr_get(x) (void *) _cairo_atomic_int_get((cairo_atomic_int_t *) x)
+_cairo_atomic_int_set_relaxed (cairo_atomic_int_t *x, int val);
+cairo_private void*
+_cairo_atomic_ptr_get(cairo_atomic_intptr_t *x);
 #else
 # define _cairo_atomic_int_get(x) (*x)
 # define _cairo_atomic_int_get_relaxed(x) (*x)
@@ -393,10 +429,10 @@ _cairo_atomic_intptr_to_voidptr (cairo_atomic_intptr_t x)
   return (void *) x;
 }
 
-static cairo_always_inline cairo_atomic_int_t
-_cairo_atomic_int_cmpxchg_return_old_fallback(cairo_atomic_int_t *x, cairo_atomic_int_t oldv, cairo_atomic_int_t newv)
+static cairo_always_inline int
+_cairo_atomic_int_cmpxchg_return_old_fallback(cairo_atomic_int_t *x, int oldv, int newv)
 {
-    cairo_atomic_int_t curr;
+    int curr;
 
     do {
         curr = _cairo_atomic_int_get (x);
@@ -406,7 +442,7 @@ _cairo_atomic_int_cmpxchg_return_old_fallback(cairo_atomic_int_t *x, cairo_atomi
 }
 
 static cairo_always_inline void *
-_cairo_atomic_ptr_cmpxchg_return_old_fallback(void **x, void *oldv, void *newv)
+_cairo_atomic_ptr_cmpxchg_return_old_fallback(cairo_atomic_intptr_t *x, void *oldv, void *newv)
 {
     void *curr;
 
@@ -441,6 +477,7 @@ _cairo_atomic_ptr_cmpxchg_return_old_fallback(void **x, void *oldv, void *newv)
 #define _cairo_status_set_error(status, err) do { \
     int ret__; \
     assert (err < CAIRO_STATUS_LAST_STATUS); \
+    assert (sizeof(*status) == sizeof(cairo_atomic_int_t)); \
     /* hide compiler warnings about cairo_status_t != int (gcc treats its as \
      * an unsigned integer instead, and about ignoring the return value. */  \
     ret__ = _cairo_atomic_int_cmpxchg ((cairo_atomic_int_t *) status, CAIRO_STATUS_SUCCESS, err); \
