@@ -93,6 +93,10 @@ cairo_format_t
 _cairo_format_from_pixman_format (pixman_format_code_t pixman_format)
 {
     switch (pixman_format) {
+    case PIXMAN_rgba_float:
+	return CAIRO_FORMAT_RGBA128F;
+    case PIXMAN_rgb_float:
+	return CAIRO_FORMAT_RGB96F;
     case PIXMAN_a8r8g8b8:
 	return CAIRO_FORMAT_ARGB32;
     case PIXMAN_x2r10g10b10:
@@ -105,11 +109,10 @@ _cairo_format_from_pixman_format (pixman_format_code_t pixman_format)
 	return CAIRO_FORMAT_A1;
     case PIXMAN_r5g6b5:
 	return CAIRO_FORMAT_RGB16_565;
-#if PIXMAN_VERSION >= PIXMAN_VERSION_ENCODE(0,22,0)
     case PIXMAN_r8g8b8a8: case PIXMAN_r8g8b8x8:
-#endif
-#if PIXMAN_VERSION >= PIXMAN_VERSION_ENCODE(0,27,2)
     case PIXMAN_a8r8g8b8_sRGB:
+#if HAS_PIXMAN_r8g8b8_sRGB
+    case PIXMAN_r8g8b8_sRGB:
 #endif
     case PIXMAN_a8b8g8r8: case PIXMAN_x8b8g8r8: case PIXMAN_r8g8b8:
     case PIXMAN_b8g8r8:   case PIXMAN_b5g6r5:
@@ -127,9 +130,7 @@ _cairo_format_from_pixman_format (pixman_format_code_t pixman_format)
     case PIXMAN_a2b10g10r10:
     case PIXMAN_x2b10g10r10:
     case PIXMAN_a2r10g10b10:
-#if PIXMAN_VERSION >= PIXMAN_VERSION_ENCODE(0,22,0)
     case PIXMAN_x14r6g6b6:
-#endif
     default:
 	return CAIRO_FORMAT_INVALID;
     }
@@ -182,7 +183,7 @@ _cairo_image_surface_create_for_pixman_image (pixman_image_t		*pixman_image,
 {
     cairo_image_surface_t *surface;
 
-    surface = _cairo_malloc (sizeof (cairo_image_surface_t));
+    surface = _cairo_calloc (sizeof (cairo_image_surface_t));
     if (unlikely (surface == NULL))
 	return _cairo_surface_create_in_error (_cairo_error (CAIRO_STATUS_NO_MEMORY));
 
@@ -244,6 +245,24 @@ _pixman_format_from_masks (cairo_format_masks_t *masks,
     return TRUE;
 }
 
+/* Convenience function to convert #cairo_dither_t into #pixman_dither_t */
+static pixman_dither_t
+_cairo_dither_to_pixman_dither (cairo_dither_t dither)
+{
+    switch (dither) {
+    case CAIRO_DITHER_FAST:
+        return PIXMAN_DITHER_FAST;
+    case CAIRO_DITHER_GOOD:
+        return PIXMAN_DITHER_GOOD;
+    case CAIRO_DITHER_BEST:
+        return PIXMAN_DITHER_BEST;
+    case CAIRO_DITHER_NONE:
+    case CAIRO_DITHER_DEFAULT:
+    default:
+        return PIXMAN_DITHER_NONE;
+    }
+}
+
 /* A mask consisting of N bits set to 1. */
 #define MASK(N) ((1UL << (N))-1)
 
@@ -322,6 +341,12 @@ _cairo_format_to_pixman_format_code (cairo_format_t format)
     case CAIRO_FORMAT_RGB16_565:
 	ret = PIXMAN_r5g6b5;
 	break;
+    case CAIRO_FORMAT_RGB96F:
+	ret = PIXMAN_rgb_float;
+	break;
+    case CAIRO_FORMAT_RGBA128F:
+	ret = PIXMAN_rgba_float;
+	break;
     case CAIRO_FORMAT_ARGB32:
     case CAIRO_FORMAT_INVALID:
     default:
@@ -401,9 +426,8 @@ cairo_image_surface_create (cairo_format_t	format,
     return _cairo_image_surface_create_with_pixman_format (NULL, pixman_format,
 							   width, height, -1);
 }
-slim_hidden_def (cairo_image_surface_create);
 
-    cairo_surface_t *
+cairo_surface_t *
 _cairo_image_surface_create_with_content (cairo_content_t	content,
 					  int			width,
 					  int			height)
@@ -439,7 +463,7 @@ _cairo_image_surface_create_with_content (cairo_content_t	content,
  *
  * Since: 1.6
  **/
-    int
+int
 cairo_format_stride_for_width (cairo_format_t	format,
 			       int		width)
 {
@@ -456,7 +480,6 @@ cairo_format_stride_for_width (cairo_format_t	format,
 
     return CAIRO_STRIDE_FOR_WIDTH_BPP (width, bpp);
 }
-slim_hidden_def (cairo_format_stride_for_width);
 
 /**
  * cairo_image_surface_create_for_data:
@@ -503,7 +526,7 @@ slim_hidden_def (cairo_format_stride_for_width);
  *
  * Since: 1.0
  **/
-    cairo_surface_t *
+cairo_surface_t *
 cairo_image_surface_create_for_data (unsigned char     *data,
 				     cairo_format_t	format,
 				     int		width,
@@ -539,7 +562,6 @@ cairo_image_surface_create_for_data (unsigned char     *data,
 							   width, height,
 							   stride);
 }
-slim_hidden_def (cairo_image_surface_create_for_data);
 
 /**
  * cairo_image_surface_get_data:
@@ -571,7 +593,6 @@ cairo_image_surface_get_data (cairo_surface_t *surface)
 
     return image_surface->data;
 }
-slim_hidden_def (cairo_image_surface_get_data);
 
 /**
  * cairo_image_surface_get_format:
@@ -595,7 +616,6 @@ cairo_image_surface_get_format (cairo_surface_t *surface)
 
     return image_surface->format;
 }
-slim_hidden_def (cairo_image_surface_get_format);
 
 /**
  * cairo_image_surface_get_width:
@@ -619,7 +639,6 @@ cairo_image_surface_get_width (cairo_surface_t *surface)
 
     return image_surface->width;
 }
-slim_hidden_def (cairo_image_surface_get_width);
 
 /**
  * cairo_image_surface_get_height:
@@ -643,7 +662,6 @@ cairo_image_surface_get_height (cairo_surface_t *surface)
 
     return image_surface->height;
 }
-slim_hidden_def (cairo_image_surface_get_height);
 
 /**
  * cairo_image_surface_get_stride:
@@ -671,9 +689,8 @@ cairo_image_surface_get_stride (cairo_surface_t *surface)
 
     return image_surface->stride;
 }
-slim_hidden_def (cairo_image_surface_get_stride);
 
-    cairo_format_t
+cairo_format_t
 _cairo_format_from_content (cairo_content_t content)
 {
     switch (content) {
@@ -689,12 +706,14 @@ _cairo_format_from_content (cairo_content_t content)
     return CAIRO_FORMAT_INVALID;
 }
 
-    cairo_content_t
+cairo_content_t
 _cairo_content_from_format (cairo_format_t format)
 {
     switch (format) {
+    case CAIRO_FORMAT_RGBA128F:
     case CAIRO_FORMAT_ARGB32:
 	return CAIRO_CONTENT_COLOR_ALPHA;
+    case CAIRO_FORMAT_RGB96F:
     case CAIRO_FORMAT_RGB30:
 	return CAIRO_CONTENT_COLOR;
     case CAIRO_FORMAT_RGB24:
@@ -712,10 +731,14 @@ _cairo_content_from_format (cairo_format_t format)
     return CAIRO_CONTENT_COLOR_ALPHA;
 }
 
-    int
+int
 _cairo_format_bits_per_pixel (cairo_format_t format)
 {
     switch (format) {
+    case CAIRO_FORMAT_RGBA128F:
+	return 128;
+    case CAIRO_FORMAT_RGB96F:
+	return 96;
     case CAIRO_FORMAT_ARGB32:
     case CAIRO_FORMAT_RGB30:
     case CAIRO_FORMAT_RGB24:
@@ -922,6 +945,8 @@ _cairo_image_surface_paint (void			*abstract_surface,
 			    const cairo_clip_t		*clip)
 {
     cairo_image_surface_t *surface = abstract_surface;
+    pixman_dither_t pixman_dither = _cairo_dither_to_pixman_dither (source->dither);
+    pixman_image_set_dither (surface->pixman_image, pixman_dither);
 
     TRACE ((stderr, "%s (surface=%d)\n",
 	    __FUNCTION__, surface->base.unique_id));
@@ -1227,6 +1252,9 @@ _cairo_image_compute_color (cairo_image_surface_t      *image)
     int x, y;
     cairo_image_color_t color;
 
+    if (image->width == 0 || image->height == 0)
+	return CAIRO_IMAGE_IS_MONOCHROME;
+
     if (image->format == CAIRO_FORMAT_A1)
 	return CAIRO_IMAGE_IS_MONOCHROME;
 
