@@ -726,42 +726,6 @@ _has_required_cairo_version (const char *str)
 			     CAIRO_VERSION_ENCODE (major, minor, micro));
 }
 
-static cairo_bool_t
-_has_required_ghostscript_version (const char *str)
-{
-#if ! CAIRO_CAN_TEST_PS_SURFACE
-    return TRUE;
-#endif
-
-    str += 2; /* advance over "gs" */
-
-    return TRUE;
-}
-
-static cairo_bool_t
-_has_required_poppler_version (const char *str)
-{
-#if ! CAIRO_CAN_TEST_PDF_SURFACE
-    return TRUE;
-#endif
-
-    str += 7; /* advance over "poppler" */
-
-    return TRUE;
-}
-
-static cairo_bool_t
-_has_required_rsvg_version (const char *str)
-{
-#if ! CAIRO_CAN_TEST_SVG_SURFACE
-    return TRUE;
-#endif
-
-    str += 4; /* advance over "rsvg" */
-
-    return TRUE;
-}
-
 #define TEST_SIMILAR	0x1
 #define TEST_OFFSET	0x2
 #define TEST_SCALE	0x4
@@ -905,30 +869,6 @@ main (int argc, char **argv)
 		else
 		    goto TEST_SKIPPED;
 	    }
-
-	    str = strstr (requirements, "gs");
-	    if (str != NULL && ! _has_required_ghostscript_version (str)) {
-		if (runner.list_only)
-		    goto TEST_NEXT;
-		else
-		    goto TEST_SKIPPED;
-	    }
-
-	    str = strstr (requirements, "poppler");
-	    if (str != NULL && ! _has_required_poppler_version (str)) {
-		if (runner.list_only)
-		    goto TEST_NEXT;
-		else
-		    goto TEST_SKIPPED;
-	    }
-
-	    str = strstr (requirements, "rsvg");
-	    if (str != NULL && ! _has_required_rsvg_version (str)) {
-		if (runner.list_only)
-		    goto TEST_NEXT;
-		else
-		    goto TEST_SKIPPED;
-	    }
 	}
 
 	if (runner.list_only) {
@@ -943,7 +883,7 @@ main (int argc, char **argv)
 	if (ctx.test->preamble != NULL) {
 	    status = _cairo_test_runner_preamble (&runner, &ctx);
 	    if (getenv ("CAIRO_TEST_UGLY_HACK_TO_IGNORE_PS_FAILURES")) {
-		if (strcmp (ctx.test_name, "ps-eps") == 0 || strcmp (ctx.test_name, "fallback-resolution") == 0) {
+		if (strcmp (ctx.test_name, "ps-eps") == 0) {
 		    if (status == CAIRO_TEST_FAILURE) {
 			cairo_test_log (&ctx, "Turning FAIL into XFAIL due to env\n");
 			fprintf (stderr, "Turning FAIL into XFAIL due to env\n");
