@@ -34,9 +34,10 @@
 
 #include "config.h"
 
-#include "cairo-script-private.h"
 #include "cairo.h"
 
+#include "cairo-script-private.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -635,7 +636,6 @@ cairo_script_interpreter_reference (csi_t *ctx)
     ctx->ref_count++;
     return ctx;
 }
-slim_hidden_def (cairo_script_interpreter_reference);
 
 cairo_status_t
 cairo_script_interpreter_finish (csi_t *ctx)
@@ -684,7 +684,6 @@ cairo_script_interpreter_destroy (csi_t *ctx)
 
     return status;
 }
-slim_hidden_def (cairo_script_interpreter_destroy);
 
 cairo_status_t
 cairo_script_interpreter_translate_stream (FILE *stream,
